<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use GuzzleHttp\Client;
use Guzzle\Http\Exception\ClientErrorResponseException;
use Illuminate\Support\Facades\Session;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Facades\Storage;

class CourseController extends Controller
{
  // Guzzle Http Client
  private $client;

  public function __construct()
  {
    $this->client = new Client(
      [
        'base_uri' => config('app.api_url'),
        'headers' => [
          'Content-Type' => 'application/json',
          'Accept' => 'application/json',
          'X-Token' => 'xnRxnTHfPlAvfvpF8j5rTcLXKVmkphcz9DwFPZjlQ9DS8vK16mEJfjQlEjPlkZ5e'
        ]
      ]
    );
  }

  /**
   * Display a listing of the resource.
   *
   * @return \Illuminate\Http\Response
   */
  public function index()
  {
    $response = json_decode($this->client->request('GET', 'Courses')->getBody());

    return view('course.index')->with([
      'courses' => $response
    ]);
  }

  /**
   * Show the form for creating a new resource.
   *
   * @return \Illuminate\Http\Response
   */
  public function create()
  {
    return view('course.create');
  }

  /**
   * Store a newly created resource in storage.
   *
   * @param  \Illuminate\Http\Request  $request
   * @return \Illuminate\Http\Response
   */
  public function store(Request $request)
  {
    $validator = Validator::make($request->all(), [
      'name' => 'required|string|max:191',
      'short_code' => 'required|string|max:3',
      'image' => 'required|image',
      'cost_in_points' => 'integer|min:0|nullable',
      'cost_in_gems' => 'integer|min:0|nullable',
      'description' => 'string|nullable'
    ]);

    if ($validator->fails()) {
      Session::flash('error', $validator->messages()->first());
      return redirect()->back()->withErrors($validator)->withInput();
    }

    $enabled = false;
    if ($request->enabled == 1) {
      $enabled = true;
    }

    $imageNameWithExt = $request->file('image')->getClientOriginalName();
    $imageName = pathInfo($imageNameWithExt, PATHINFO_FILENAME);
    $extension = $request->file('image')->getClientOriginalExtension();
    $imageNameToStore = $imageName.'_'.time().'.'.$extension;

    $body = [
      'name' => $request->name,
      'short_code' => $request->short_code,
      'image' => $imageNameToStore,
      'cost_in_points' => $request->cost_in_points,
      'cost_in_gems' => $request->cost_in_gems,
      'description' => $request->description,
      'enabled' => $enabled
    ];

    try {
      $response = $this->client->request('POST', 'Courses', ['json' => $body]);

      if ($response->getStatusCode() == 200) {
        $path = $request->file('image')
        ->storeAs('public/course/images/', $imageNameToStore);
      }
    } catch (\Exception $exception) {
        // return $exception->getMessage();
        return redirect()->back()->withErrors(['error' => $exception->getMessage()])->withInput();
    }

    return redirect()->route('course.index');
  }

  /**
   * Display the specified resource.
   *
   * @param  int  $id
   * @return \Illuminate\Http\Response
   */
  public function show($id)
  {
    $response =json_decode($this->client->request('GET', 'Courses/'.$id)->getBody());
    $levels =json_decode($this->client->request('GET', 'Courses/'.$id.'/levels')->getBody());

    return view('course.show')->with([
      'course' => $response,
      'levels' => $levels
    ]);
  }

  /**
   * Show the form for editing the specified resource.
   *
   * @param  int  $id
   * @return \Illuminate\Http\Response
   */
  public function edit($id)
  {
    $response = json_decode($this->client->request('GET', 'Courses/'.$id)->getBody());

    return view('course.edit')->with([
      'course' => $response
    ]);
  }

  /**
   * Update the specified resource in storage.
   *
   * @param  \Illuminate\Http\Request  $request
   * @param  int  $id
   * @return \Illuminate\Http\Response
   */
  public function update(Request $request, $id)
  {
    $course = $response = json_decode($this->client->request('GET', 'Courses/'.$id)->getBody());

    $validator = Validator::make($request->all(), [
      'name' => 'required|string|max:191',
      'short_code' => 'required|string|max:3',
      'image' => 'image',
      'cost_in_points' => 'integer|min:0|nullable',
      'cost_in_gems' => 'integer|min:0|nullable',
      'description' => 'string|nullable'
    ]);

    if ($validator->fails()) {
      Session::flash('error', $validator->messages()->first());
      return redirect()->back()->withErrors($validator)->withInput();
    }

    $enabled = false;
    if ($request->enabled == 1) {
      $enabled = true;
    }

    $imageNameToStore = $course->image;

    if ($request->hasFile('image')) {
      $imageNameWithExt = $request->file('image')->getClientOriginalName();
      $imageName = pathInfo($imageNameWithExt, PATHINFO_FILENAME);
      $extension = $request->file('image')->getClientOriginalExtension();
      $imageNameToStore = $imageName.'_'.time().'.'.$extension;
    }

    $body = [
      'name' => $request->name,
      'short_code' => $request->short_code,
      'image' => $imageNameToStore,
      'cost_in_points' => $request->cost_in_points,
      'cost_in_gems' => $request->cost_in_gems,
      'description' => $request->description,
      'enabled' => $enabled
    ];

    try {
      $response = $this->client->request('PUT', 'Courses/'.$id, ['json' => $body]);

      if ($response->getStatusCode() == 200) {
        if ($imageNameToStore != $course->image) {
          Storage::delete('public/course/images/'.$course->image);
          $path = $request->file('image')->storeAs('public/course/images/', $imageNameToStore);
        }
      }
    } catch (\Exception $exception) {
        // return $exception->getMessage();
        return redirect()->back()->withErrors(['error' => $exception->getMessage()])->withInput();
    }

    return redirect()->route('course.index');
  }

  /**
   * Remove the specified resource from storage.
   *
   * @param  int  $id
   * @return \Illuminate\Http\Response
   */
  public function destroy($id)
  {
    $course = json_decode($this->client->request('GET', 'Courses/'.$id)->getBody());

    $response = $this->client->request('DELETE', 'Courses/'.$id);

    if ($response->getStatusCode() == 200) {
      Storage::delete('public/course/images/'.$course->image);
    }

    return redirect()->route('course.index');
  }
}
